/****************************************************************************

    Change : Substitutes sequences in a wordlist.
    Copyright (C) 1994  Sbragion Denis

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    You can contact the author on Internet at the following addresses :

      sbragion@ghost.sm.dsi.unimi.it (preferred)
      denissbr@ipo.sm.dsi.unimi.it

    or through standard mail at :

      Sbragion Denis
      Via E. Fermi 18
      20046 Biassono (MI)
        ITALY

****************************************************************************/

/****************************************************************************
  Progetto    : Programmi gestione wordlists.
  File        : Change.c
  Autore      : Sbragion Denis
  Descrizione : Sostituisce delle sequenze in una wordlist.
  Revisioni   :
  16/6/94     : Prima stesura.
  27/6/94     : Cambiamento del nome da Subst a Change. Correzione di alcuni
                messaggi di errore. Riduzione dell' occupazione dello stack.
****************************************************************************/

#include "boolean.h"
#include "strcomp.h"
#include <string.h>
#include <stddef.h>

#define MaxLineLength (MaxStringLength + 1)
#define WriteModulus 1000
#define MaxSubstitionLength 16
#define MaxSubstitionCount 20

void Substitute(char * D,char * S,char * SI,char * SO)
  {
    char * SP = S;
    size_t SIL = strlen(SI);

    D[0] = '\0';

    do
      {
        SP = strstr(S,SI);

        if (SP == NULL)
          strcat(D,S);
        else
          {
            strncat(D,S,(size_t) (SP - S));
            strcat(D,SO);
            S = SP + SIL;
          }
      }
    while(SP != NULL);
  }

int main(int argc,char * argv[])
  {
    FILE * InFile;
    FILE * OutFile;
    static char Str1[MaxLineLength +
      (MaxSubstitionLength * MaxSubstitionCount)];
    static char Str2[MaxLineLength +
      (MaxSubstitionLength * MaxSubstitionCount)];
    char * SourceString = Str1;
    char * DestinationString = Str2;
    char * SwapPointer;
    unsigned long int LineCount = 0;
    unsigned int I;

    if (argc < 5)
      {
        puts("Change : Substitutes sequences in a wordlist.");
        puts("Usage : Change InputFile OutputFile OldStr NewStr "
          "[OldStr NewStr] ...");
        return((int) Ok);
      }

    if (argc > (2 * MaxSubstitionCount + 3))
      {
        puts("Error : Too many substitutions.");
        puts("Usage : Change InputFile OutputFile OldStr NewStr "
          "[OldStr NewStr] ...");
        return((int) Error);
      }

    if ((argc % 2) == 0)
      {
        puts("Error : Wrong number of input parameters.");
        puts("Usage : Change InputFile OutputFile OldStr NewStr "
          "[OldStr NewStr] ...");
        return((int) Error);
      }

    for (I = 3;I < argc;I++)
      if (strlen(argv[I]) > MaxSubstitionLength)
        {
          puts("Error : Substitution string too long.");
          puts("Usage : Change InputFile OutputFile OldStr NewStr "
            "[OldStr NewStr] ...");
          return((int) Error);
        }

    if ((InFile = fopen(argv[1],"rt")) == NULL)
      {
        puts("Error : Can't open input file.");
        puts("Usage : Change InputFile OutputFile OldStr NewStr "
          "[OldStr NewStr] ...");
        return((int) Error);
      }

    if ((OutFile = fopen(argv[2],"wt")) == NULL)
      {
        puts("Error : Can't open output file.");
        puts("Usage : Change InputFile OutputFile OldStr NewStr "
          "[OldStr NewStr] ...");
        fclose(InFile);
        return((int) Error);
      }

    printf("Changing : ");
    fflush(stdout);

    while (fgets(SourceString,MaxLineLength,InFile) != NULL)
      {
        StripEndOfLine(SourceString);

        for (I = 3;I < argc;I += 2)
          {
            Substitute(DestinationString,SourceString,argv[I],argv[I+1]);
            SwapPointer = SourceString;
            SourceString = DestinationString;
            DestinationString = SwapPointer;
          }

        if (fprintf(OutFile,"%s\n",SourceString) == EOF)
          {
            puts("\nError : Can't write to output file.");
            fclose(InFile);
            fclose(OutFile);
            return((int) Error);
          }

        LineCount++;
        if ((LineCount % WriteModulus) == 0)
          {
            printf("*");
            fflush(stdout);
          }
      }

    fclose(InFile);
    fclose(OutFile);

    printf("\n%lu Lines.\n",LineCount);
    puts("Done.");
    return((int) Ok);
  }

/***************************************************************************/
